==> makeen-expressionengine-0.0.51.tgz: package/src/functions/globals/select.ts <==
﻿import { mfunction } from './../mfunction'
import { namedSelection } from './../namedSelection'
import expressionData from '../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { groupDataContext } from './../../functions/groupDataContext'
import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import { IEnumerable } from 'typescript-dotnet-commonjs/System/Collections/Enumeration/IEnumerable'
import { KeyValuePair } from 'typescript-dotnet-commonjs/System/KeyValuePair'
import { operationPart } from './../../compiler/parts/operationPart'
import { selection } from '../selection'
import { outputVariables } from '../../utils/outputVariables'

export class select extends mfunction {

    public /* override */ get minParameters(): number {
        return 1;
    }

    public /* override */ get maxParameters(): number {
        return -1;
    }

    public namedSelection: boolean | undefined;

    public invoke(expressionData: expressionData, parameters: List<expressionPart>, /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        let tempData: expressionData = expressionData.toSubEx();
       resolved.resolved = true;
        let isNamedSelection: boolean = (this.namedSelection || parameters.any((t => t instanceof operationPart && (<operationPart>(t)).operator?.name == "@")));
        if ((obj instanceof List)) {
            let result: List<Object> = new List<Object>();
            let done: boolean = false;
            while (!done) {
                result.clear();
                //  Clear();
                try {
                    for (let item of (<List<Object>>(obj)).toArray()) {
                        let values: Object[] = new Array(parameters.count);
                        tempData.contexts.push(item);
                        for (let i: number = 0; (i < parameters.count); i++) {
                            let subResolved: outputVariables = new outputVariables();
                            let value: Object | undefined = parameters.get(i).resolve(tempData, /* out */subResolved, /* running:*/ undefined);
                            if (!subResolved.resolved) {
                                value = undefined;
                            }

                            // if ((isNamedSelection
                            //     && !(value instanceof KeyValuePair))) {
                            //     value = new KeyValuePair<string, Object>(parameters[i].SimpleName, value);
                            // }

                            values[i] = value as Object;
                        }

                        if (isNamedSelection) {
                            result.add(new namedSelection(values));
                        }
                        else if ((values.length == 1)) {
                            result.add(values[0]);
                        }
                        else {
                            result.add(new selection(values));
                        }

                        tempData.contexts.pop();
                    }

                    done = true;
                }
                catch (e) {
                    let d= e;
                }

            }

            return result;
        }
        else {
            return obj;
        }

    }
}

==> makeen-expressionengine-0.0.63.tgz: package/src/functions/globals/select.ts <==
﻿import { mfunction } from './../mfunction'
import { namedSelection } from './../namedSelection'
import expressionData from '../../expressionData'
import { expressionPart } from './../../compiler/parts/expressionPart'
import { groupDataContext } from './../../functions/groupDataContext'
import { List } from 'typescript-dotnet-commonjs/System/Collections/List'
import { operationPart } from './../../compiler/parts/operationPart'
import { selection } from '../selection'
import { outputVariables } from '../../utils/outputVariables'

export class select extends mfunction {

     /* override */ get minParameters(): number {
        return 1;
    }

     /* override */ get maxParameters(): number {
        return -1;
    }

     namedSelection: boolean | undefined;

     invoke(expressionData: expressionData, parameters: expressionPart[], /* out */resolved: outputVariables, running: groupDataContext | undefined): Object {
        let obj: Object = expressionData.contexts.peek();
        let tempData: expressionData = expressionData.toSubEx();
       resolved.resolved = true;
        let isNamedSelection: boolean = (this.namedSelection || parameters.some((t: expressionPart) => { return t instanceof operationPart && (<operationPart>(t)).operator?.name == "@"}));
        if ((obj instanceof List)) {
            let result: List<Object> = new List<Object>();
            let done: boolean = false;
            while (!done) {
                result.clear();
                //  Clear();
                try {
                    for (let item of (<List<Object>>(obj)).toArray()) {
                        let values: Object[] = new Array(parameters.length);
                        tempData.contexts.push(item);
                        for (let i: number = 0; (i < parameters.length); i++) {
                            let subResolved: outputVariables = new outputVariables();
                            let value: Object | undefined = parameters[i].resolve(tempData, /* out */subResolved, /* running:*/ undefined);
                            if (!subResolved.resolved) {
                                value = undefined;
                            }

                            // if ((isNamedSelection
                            //     && !(value instanceof KeyValuePair))) {
                            //     value = new KeyValuePair<string, Object>(parameters[i].SimpleName, value);
                            // }

                            values[i] = value as Object;
                        }

                        if (isNamedSelection) {
                            result.add(new namedSelection(values));
                        }
                        else if ((values.length == 1)) {
                            result.add(values[0]);
                        }
                        else {
                            result.add(new selection(values));
                        }

                        tempData.contexts.pop();
                    }

                    done = true;
                }
                catch (e) {
                    throw e;
                }

            }

            return result;
        }
        else {
            return obj;
        }

    }
}
